from pwn import *
from curve25519 import Private,Public
import nacl.secret
from binascii import hexlify, unhexlify
import struct
import sys

# curve25519: https://github.com/agl/curve25519-donna
# nacl: https://github.com/pyca/pynacl

# ===== HELPER =====
# Little endian conversion in python2
def to_bytes(n):
	out = ''
	for i in xrange(32):
		n,c = divmod(n, 256)
		out += chr(c)
	return out

# Mirror ReadBin() from challenge.py
def ReadBin(remote):
	return unhexlify(remote.recvline().strip())

# Mirror WriteBin() from challenge.py
def WriteBin(remote, data):
	remote.sendline(hexlify(data))
# ==================

# This is the prime used for Curve25519 (via https://en.wikipedia.org/wiki/Curve25519)
PRIME = 2**255-19 		


# Note: You can run client/server locally for testing with cmds below.
#   Also remember to create "password.txt" and "flag.txt" files.
#
#   ncat -lk -p 13706 -e challenge.py
#   ncat -lk -p 13707 -e challenge.py
#s = remote('127.0.0.1', 13706)					# Local server		
#c = remote('127.0.0.1', 13707)					# Local client
s = remote('mitm.ctfcompetition.com',1337)  	# Challenge server
c = remote('mitm.ctfcompetition.com',1337)  	# Challenge client

# Select server/client respectively
s.sendline('s')									# [Server] Talk to server
c.sendline('c')									# [Client] Talk to client


# ===== HANDSHAKE =====
# Ensure that the server and client pass the handshake

# Get public keys and nonces		
s_publickey = ReadBin(s)							# [Server] Public key
s_nonce = ReadBin(s)								# [Server] Nonce
 
c_publickey = ReadBin(c)							# [Client] Public key
c_nonce = ReadBin(c)								# [Client] Nonce

# Generate a weak key that's not filtered out by challenge.py
#   via https://gist.github.com/jedisct1/39f8dee6e38b12bb34f5
# Picked (p+1) for its small order
weak_key = to_bytes(PRIME+1)
							
# Give server and client the weak key & each others' nonces
WriteBin(s, weak_key)								# [Server] Give weak key
WriteBin(s, c_nonce)								# [Server] Give client nonce
WriteBin(c, weak_key)								# [Client] Give weak key
WriteBin(c, s_nonce)								# [Client] Give server nonce
			
# Receive the server and client proofs		
s_proof = ReadBin(s)								# [Server] Get proof
c_proof = ReadBin(c)								# [Client] Get proof
					
# Send the server and client proofs to each other
WriteBin(s, c_proof)								# [Server] Give client proof
WriteBin(c, s_proof)								# [Client] Give server proof


# ===== CONFIRM AUTHENTICATED =====
# Server will give AUTHENTICATED status

s_status = ReadBin(s)								# [Server] Get AUTHENTICATED


# ===== MAKE PRIVATE KEY =====
# Generate my private key in the same way the client/server would do so in
#   challenge.py, using our weak public key and a generated private key.

myPrivateKey = Private()									
sharedKey = myPrivateKey.get_shared_key(Public(weak_key))	
mySecretBox = nacl.secret.SecretBox(sharedKey)

print "[Server] Status: " + mySecretBox.decrypt(s_status)


# ===== SEND CMDS TO SERVER =====
# At this point, the client-server handshake succeeded so the server believes
#   it is connected to the client. We can ignore the client and send data to 
#   the server as if we're the client using our private key.

WriteBin(s, mySecretBox.encrypt(b"getflag"))		# Get flage
print "[Flag]: " + mySecretBox.decrypt(ReadBin(s))				# Print flage

